/**
    FitsColorControl.c: Color handling

    Copyright 1992 Barry Solomon 

    Permission to use, copy, modify, distribute this software is hereby 
    granted without fee, provided that this Copyright notice appear in all 
    copies.  It is provided "as is" without express or implied warranty.       
 
    Revision
    01/21/92	BDS	Created.

**/

#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>

#include "FitsMenuConstants.h"
#include "FitsColorDefs.h"
#include "FitsImageDefs.h"
#include "FitsDefs.h"
#include "FitsOctreeDefs.h"

XColor LocColors[256];

/*************************************************** set_color_range */
set_color_range( color)
/**
**/
    char  color; /* r, g, or b */
{
    int  i, intensity;
/*--------------------------------------------------------------------*/
    for (i=MINCELL;i<numcells;i++) {

        LocColors[i].pixel = i;
        LocColors[i].red   = 0;
        LocColors[i].green = 0;
        LocColors[i].blue  = 0;
        LocColors[i].flags = DoRed | DoGreen | DoBlue;

        intensity = (i * 65535) / numcells;

        switch ( color) {
            case 'r':
                LocColors[i].red   = intensity;
                break;
            case 'g':
                LocColors[i].green = intensity;
                break;
            case 'b':
                LocColors[i].blue  = intensity;
                break;
        }

    }

    XStoreColors( display, mycmap, LocColors, numcells);
    XInstallColormap( display, mycmap);
    XFlush( display);

} /* end set_color_range() */

/************************************************* load_greyscale */
load_greyscale()
/**
**/
{
    int       i;
/*--------------------------------------------------------------------*/

    for (i=MINCELL;i<numcells;i++) {
        LocColors[i].pixel = i;
        LocColors[i].red   = (i * 65535) / numcells;
        LocColors[i].green = (i * 65535) / numcells;
        LocColors[i].blue  = (i * 65535) / numcells;
        LocColors[i].flags = DoRed | DoGreen | DoBlue;
    }

    XStoreColors( display, mycmap, LocColors, numcells);
    XFlush( display);

} /* end greyscale() */

/*************************************************** load_color_range */
load_color_range()
/**
**/
{
    int       i;
/*--------------------------------------------------------------------*/

    for (i=MINCELL;i<numcells;i++) {
        LocColors[i].pixel = i;
        LocColors[i].red   = ((numcells-i  ) * 65535 ) / numcells;
        LocColors[i].green = ((         i  ) * 65535 ) / numcells;
        LocColors[i].blue  = ((numcells-i/2) * 65535 ) / numcells;
        LocColors[i].flags = DoRed | DoGreen | DoBlue;
    }

    XStoreColors( display, mycmap, LocColors, numcells);
    XInstallColormap( display, mycmap);
    XFlush( display);

} /* end load_color_range() */

/******************************************************** CalcColormap */
ErrorCode   CalcColormap( NodePtr)
/**
    Traverse the octree and load the colormap with respect to the image 
    control variables detailing how to load the colormap
**/
    struct OctreeNode *NodePtr;
{
    int             i, j;
    Logical         IsLeaf = TRUE;
/*---------------------------------------------------------------------*/

/*
 *  Is this a leaf node
 */
    for (j = 0; j < 8; j++)
	if (NodePtr->Child[j] != NULL)
	    IsLeaf = FALSE;

/*
 *  If the current depth is less than ColorLevelMax depth then set/load color
 */
    if (NodePtr->Depth < ColorLevelMax) {

	if (IsLeaf) {

	    NodePtr->Index = CurColorIndex++;

	    if (image.channel[RED] != NONE)
		colors[NodePtr->Index].red = NodePtr->Value[RED] * 65535;
	    else
		colors[NodePtr->Index].red = 0;

	    if (image.channel[GREEN] != NONE)
		colors[NodePtr->Index].green = NodePtr->Value[GREEN] * 65535;
	    else
		colors[NodePtr->Index].green = 0;

	    if (image.channel[BLUE] != NONE)
		colors[NodePtr->Index].blue = NodePtr->Value[BLUE] * 65535;
	    else
		colors[NodePtr->Index].blue = 0;

	}
	else
/*
 *	    For each child
 */
	    for (i = 0; i < 8; i++) {
		if (NodePtr->Child[i] != NULL)
		    CalcColormap(NodePtr->Child[i]);
	    }

    }

/*
 *  If the current depth is ColorLevelMax depth then set/load color
 *  note: every node at this level gets assigned a color index and loaded
 */
    if (NodePtr->Depth == ColorLevelMax) {

	NodePtr->Index = CurColorIndex++;
/*
 *	calc red color value
 */
	if (image.channel[RED] != NONE) {
	    if (IsLeaf == FALSE)
		NodePtr->Value[RED] =
		    (NodePtr->MaxVal[RED] + NodePtr->MinVal[RED]) / 2;
	    colors[NodePtr->Index].red = NodePtr->Value[RED] * 65535;
	}
	else
	    colors[NodePtr->Index].red = 0;
/*
 *	calc green color value
 */
	if (image.channel[GREEN] != NONE) {
	    if (IsLeaf == FALSE)
		NodePtr->Value[GREEN] =
		    (NodePtr->MaxVal[GREEN] + NodePtr->MinVal[GREEN]) / 2;
	    colors[NodePtr->Index].green = NodePtr->Value[GREEN] * 65535;
	}
	else
	    colors[NodePtr->Index].green = 0;
/*
 *	calc blue color value
 */
	if (image.channel[BLUE] != NONE) {
	    if (IsLeaf == FALSE)
		NodePtr->Value[BLUE] = 
		    (NodePtr->MaxVal[BLUE] + NodePtr->MinVal[BLUE]) / 2;
	    colors[NodePtr->Index].blue = NodePtr->Value[BLUE] * 65535;
	}
	else
	    colors[NodePtr->Index].blue = 0;

/*
 *	for each child...
 */
	for (i = 0; i < 8; i++) {
	    if (NodePtr->Child[i] != NULL)
		CalcColormap(NodePtr->Child[i]);
	}

    }

/*
 *  If the current depth is greater than ColorLevelMax
 *  then simply set the Index of the Node
 */
    if (NodePtr->Depth > ColorLevelMax) {

	NodePtr->Index = NodePtr->Parent->Index;
/*
 *	for each child...
 */
	for (i = 0; i < 8; i++) {
	    if (NodePtr->Child[i] != NULL)
		CalcColormap(NodePtr->Child[i]);
	}

    }

}

/******************************************************** LoadColormap */
void LoadColormap()
{
    int             NumLeavesAbove, i;
/*---------------------------------------------------------------------*/

    ColorLevelMax = 0;
    CurColorIndex = 16;
    NumAvailColors = numcells - 16;
    NumLeavesAbove = LeavesPerLevel[0];
/*
 *  Get the colors that are in the colormap
 */
    for (i = 0; i < 256; i++) {
	colors[i].pixel = i;
	colors[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(display, mycmap, colors, numcells);

    for (i = 1; i < OCTREE_MAX_NODE_DEPTH; i++) {
	if ((ChildrenPerLevel[i] + NumLeavesAbove) < NumAvailColors)
	    ColorLevelMax++;
	NumLeavesAbove += LeavesPerLevel[i];
    }
    CalcColormap(RootNode);

    image.MaxColorIndex = CurColorIndex;

    XStoreColors(display, mycmap, colors, numcells);
    XFlush(display);

    for (i = 0; i < numcells; i++) {
	SavedColors[i][RED] = colors[i].red;
	SavedColors[i][GREEN] = colors[i].green;
	SavedColors[i][BLUE] = colors[i].blue;
    }

}/* end LoadColormap() */

/****************************************************** AnimateColormap */
AnimateColormap()
{
    ErrorCode       RC;
    int             i = 0;
    XColor          LocColors[256], SaveColor;
    int             Count, ColorId, NumColors, TmpPixel;
    int             Delay;
    struct timeb    t1, t2;
/*---------------------------------------------------------------------*/
    NumColors = (CurColorIndex > 0) ? CurColorIndex : 256;

    Delay = 0;
    display_menu(ANIMATE_M);
    get_input(&Delay, DECIMAL);

/*
 *  Get the colors that are in the colormap
 */
    for (i = 0; i < 256; i++) {
	LocColors[i].pixel = i;
	LocColors[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(display, mycmap, LocColors, NumColors);

/*
 *  Cycle the colors NumCycles times
 */
    for (Count = 0; Count < (NumColors - MINCELL); Count++) {

	ftime(&t1);

	SaveColor.pixel = LocColors[NumColors - 1].pixel;
	SaveColor.red = LocColors[NumColors - 1].red;
	SaveColor.green = LocColors[NumColors - 1].green;
	SaveColor.blue = LocColors[NumColors - 1].blue;

	for (i = NumColors - 1; i > MINCELL; i--) {
	    LocColors[i].red = LocColors[i - 1].red;
	    LocColors[i].green = LocColors[i - 1].green;
	    LocColors[i].blue = LocColors[i - 1].blue;
	}

	LocColors[i].pixel = SaveColor.pixel;
	LocColors[i].red = SaveColor.red;
	LocColors[i].green = SaveColor.green;
	LocColors[i].blue = SaveColor.blue;

	XStoreColors(display, mycmap, LocColors, NumColors);
	XSync(display, 0);

	ftime(&t2);
	while ((t2.millitm - t1.millitm) < Delay)
	    ftime(&t2);

    }

    XStoreColors(display, mycmap, colors, NumColors);
    XSync(display, 0);

}

/****************************************************** AnimateColormap */
TraverseColormap()
{
    ErrorCode       RC;
    int             i = 0;
    XColor          LocColors[256], SaveColor;
    int             Count, ColorId, NumColors, NumCycles;
    int             Delay;
    struct timeb    t1, t2;
/*---------------------------------------------------------------------*/
    NumColors = (CurColorIndex > 0) ? CurColorIndex : 256;
    NumCycles = 1;
    Delay = 10;
/***
    display_menu(ANIMATE_M);
    get_input(&Delay, DECIMAL);
    display_menu(MAIN_M);
***/
/*
 *  Get the colors that are in the colormap
 */
    for (i = 0; i < 256; i++) {
	LocColors[i].pixel = i;
	LocColors[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(display, mycmap, LocColors, NumColors);

    XSynchronize(display, 1);
/*
 *  Cycle the colors NumCycles times
 */
    for (Count = 0; Count < NumCycles; Count++) {

	for (i = MINCELL; i < 256; i++) {

	    ftime(&t1);

	    SaveColor.red = LocColors[i].red;
	    SaveColor.green = LocColors[i].green;
	    SaveColor.blue = LocColors[i].blue;

	    LocColors[i].red = 65535;
	    LocColors[i].green = 65535;
	    LocColors[i].blue = 65535;

	    XStoreColor(display, mycmap, LocColors[i]);
	    XSync(display, 0);

	    LocColors[i].red = SaveColor.red;
	    LocColors[i].green = SaveColor.green;
	    LocColors[i].blue = SaveColor.blue;

	    XStoreColor(display, mycmap, LocColors[i]);
	    XSync(display, 0);

	    ftime(&t2);
	    while ((t2.millitm - t1.millitm) < Delay)
		ftime(&t2);
	}

    }

    XSynchronize(display, 0);
    XStoreColors(display, mycmap, colors, NumColors);
    XSync(display, 0);

}

/*********************************************** LoadPredefinedCMap */
void LoadPredefinedCMap()
/**
    Display a menu of all the colormaps available
    and load the one the user selects
**/
{
    char CMapType;
/*-----------------------------------------------------------------*/

    display_menu( COLOR_M);

    get_input( &CMapType, CHAR);

    switch( CMapType) {

        case 'r' : /* set a red colormap */
        case 'g' : /* set a green colormap */
        case 'b' : /* set a blue colormap */
            set_color_range( CMapType);
            break;

        case 's' : /* Load a greyscale colormap */
            load_greyscale();
            break;

        case 'd' :
             XStoreColors( display, mycmap, colors, dcms);
             XFlush( display);
            break;

        case 'm' : /* Load a colorramp and display */
            load_color_range();
            break;

    } /* end switch (colormap) */

    display_menu( MAIN_M);

 } /* end LoadPredefinedCMap() */
