/**
    FitsDisplayControl.c: Display control of the image

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Revision
    01/21/92	BDS	Created.
    03/03/92	BDS	Move RegenImage to this module.
**/

#include <stdio.h>

#include "FitsMenuConstants.h"
#include "FitsWindowDefs.h"
#include "FitsColorDefs.h"
#include "FitsOctreeDefs.h"
#include "FitsImageDefs.h"
#include "FitsDefs.h"

/************************************************** set_display_method */
void set_display_method( )
/**
    Set the method by which the fits images will be displayed
**/
{
    char        DisplayMethodCode;
/*---------------------------------------------------------------------*/
    display_menu(DISPMETHOD_M);

    get_input(&DisplayMethodCode, CHAR);

    switch (DisplayMethodCode) {
	case 'g':
	    printf("Method GRAPH not implemented - YET, using COLOUR\n");
	    break;
	case 'c':
	default:
	    break;
    }				/* end switch() */

    display_menu(MAIN_M);

}				/* end set_display_method() */

/******************************************************* DisplayImage */
DisplayImage()
{
    int         xpos, ypos, index;
    int         row, col;
    int        *ImagePixelPtr;
/*--------------------------------------------------------------------*/

    if (image.idata == NULL)
	return;

/*
 *  Set the cursor to busy
 */
    update_graphics_window(image.nrows, image.ncols);

    SetCursor(stopwatch_cursor, ALL_WINDOWS);

    ImagePixelPtr = (int *) image.idata;

/*
 *  Fill a rectagle for each color in the colorrange
 */
    for (row = 0; row < image.nrows; row++) {

	for (col = 0; col < image.ncols; col++) {
/*
 *          Calculate the position for this image cell
 */
	    xpos = (col * ImageCellWidth);
	    ypos = gwh - (row * ImageCellHeight);
/*
 *          Make sure this colormap index is within range
 */
	    index = *ImagePixelPtr;
	    if (index >= numcells)
		index = image.MaxColorIndex;
	    if (index < MINCELL)
		index = MINCELL;

/*
 *          Set the color used to draw the filled rectangle
 */
	    XSetForeground(display, gc, index);
/*
 *          Draw a filled rectangle
 */
	    XFillRectangle(display, graph_win, gc,
			   xpos, ypos, ImageCellWidth, ImageCellHeight);

	    ImagePixelPtr++;

	}			/* end for col */
    }				/* end for row */

/*
 *  Flush the output buffer
 */
    XFlush(display);

/*
 *  Set the cursor to normal
 */
    SetCursor(normal_cursor, ALL_WINDOWS);

}				/* end DisplayImage() */

/************************************************************ RegenImage */
ErrorCode RegenImage()
/**
**/
{
/*-----------------------------------------------------------------------*/
    SetCursor(stopwatch_cursor, ALL_WINDOWS);

    LoadOctree();

    OctreeCalcStats(RootNode);

    LoadColormap();

    LoadImageArray();

    DisplayImage();

    SetCursor(normal_cursor, ALL_WINDOWS);

} /* end RegenImage() */

/********************************************** DisplayControl */
ErrorCode   DisplayControl()
{
    char            CharCode;
/*-----------------------------------------------------------*/
    display_menu(DISPCONTROL_M);

    get_input(&CharCode, CHAR);

    switch (CharCode) {
    case 'd':	/* Re-Display the image */
	DisplayImage();
	break;
    case 'r':	/* Regen Image */
	RegenImage();
	break;

    }

    display_menu(MAIN_M);

}
