/**
    FitsEvents.c: Event handling

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Author Change Log
    01/21/92	BDS	Created.
    02/10/92	BDS	Added scaling (range focus))
    03/03/92	BDS	Modify CMap output to display fimage values
**/

#include <stdio.h>
#include <math.h>

#include "FitsDefs.h"
#include "FitsMenuDefs.h"
#include "FitsColorDefs.h"
#include "FitsWindowDefs.h"
#include "FitsImageDefs.h"
#include "FitsOctreeDefs.h"

char message[133];
extern double strtod_by_me();

/*********************************************************** get_input */
get_input( input, type)
/**
    Wait for the next KeyPress and defer all other events
    and return keystring[0] in 'input'
**/
    char *input;
    int  type;
{
    XEvent          ev;
    char            string[20], *keystring;
    KeySym          key;
    Bool            done = FALSE;
    char            lstring[133];
    int             lvalue = 0;
/*---------------------------------------------------------------------*/
    strcpy(lstring, "");

    while (!done) {

	XMaskEvent(display, KeyPressMask, &ev);

	XLookupString(&ev, string, 20, &key, NULL);
	keystring = XKeysymToString(key);

	switch (type) {

	case CHAR:
	    if ((strlen(keystring) < 2) || (!strcmp(keystring, "Return"))) {
		*input = keystring[0];
		done = TRUE;
	    }
	    break;

	case DECIMAL:

	    if (!strcmp(keystring, "Return")) {
		if (strcmp(lstring, ""))
		    *(int *) input = lvalue;
		done = TRUE;
	    }

	    if (!strcmp(keystring, "BackSpace") && (strlen(lstring) > 0)) {
		lstring[strlen(lstring) - 1] = '\0';
		lvalue /= 10;
	    }

	    if ((keystring[0] >= '0') && (keystring[0] <= '9')) {
		lvalue = 10 * lvalue + (keystring[0] - '0');
		sprintf(lstring, "%d", lvalue);
	    }
	    break;

	case FLOAT:

	    if (!strcmp(keystring, "Return")) {
		if (strcmp(lstring, ""))
		    *(float *) input = (double) strtod_by_me(lstring);
		done = TRUE;
	    }
	    if (!strcmp(keystring, "BackSpace") && (strlen(lstring) > 0)) {
		lstring[strlen(lstring) - 1] = '\0';
	    }
	    if ((keystring[0] >= '0') && (keystring[0] <= '9')) {
		sprintf(lstring, "%s%s\0", lstring, keystring);
	    }
	    if (!strcmp(keystring, "minus")) {
		sprintf(lstring, "%s-\0", lstring);
	    }
	    if (!strcmp(keystring, "period")) {
		sprintf(lstring, "%s.\0", lstring);
	    }

	    break;


	case STRING:

	    if (!strcmp(keystring, "Return")) {
		strcpy(input, lstring);
		done = TRUE;
	    }

	    if (!strcmp(keystring, "BackSpace") && (strlen(lstring) > 0)) {
		lstring[strlen(lstring) - 1] = '\0';
	    }
	    if (!strcmp(keystring, "period")) {
		sprintf(lstring, "%s.\0", lstring);
	    }
	    if (!strcmp(keystring, "slash")) {
		sprintf(lstring, "%s/\0", lstring);
	    }
	    if (strlen(keystring) < 2) {
		sprintf(lstring, "%s%s\0", lstring, keystring);
	    }

	    break;

	}	/* end switch(type) */

	XSetForeground(display, gc, 0);
	XFillRectangle(display, output_win, gc,
		       10, 10 + text_height * menu[cur_menu].num_lines,
		       oww, owh);

	XDrawImageString(display, output_win, gc,
			 10, 20 + text_height * menu[cur_menu].num_lines,
			 lstring, strlen(lstring));

    }	/* end while(!done) */

    display_menu(MAIN_M);

}/* end get_input() */

/************************************************** dispatch_key_event */
dispatch_key_event( key)
/**
**/
    char  *key;
{
/*-------------------------------------------------------------------- */

    if (strlen(key) == 1)

	switch (key[0]) {

	case 'A':
	    TraverseColormap();
	    break;

	case 'a':
	    AnimateColormap();
	    break;

	case 'c':	/* Load a colormap */
	    LoadPredefinedCMap();
	    break;

	case 'd':
	    DisplayControl();
	    break;

	case 'f':	/* Change the image processing translation */
	    SetFunction();
	    break;

	case 'i':
	    ImageControl();
	    break;

	case 'p':	/* Temp - Print the octree leafs */
	    OctreePrint(RootNode);
	    break;

	case 'q':	/* quit this program */
	    exit_program = TRUE;
	    XCloseDisplay(display);
	    break;

	default:
	    display_menu(MAIN_M);

	}	/* end switch(keystring[0]) */

}/* end dispatch_key_event() */

/********************************************************* event_loop */
void event_loop()
{
    XEvent          event;
    char            string[20], *keystring;
    KeySym          key;
    Bool            update_picture = FALSE;
    Bool            First = TRUE;
    int             i, j, num, bgcolor;
    static XColor   SavedColor;
    unsigned int    clr_mask = DoRed | DoGreen | DoBlue;
/*--------------------------------------------------------------------*/
    message[0] = '\0';

    while (!exit_program) {

	if (!XPending(display) && update_picture) {
	    if (First)
		First = FALSE;
	    else
		DisplayImage();
	    update_picture = FALSE;
	}

	XNextEvent(display, &event);

	switch (event.type) {

	case ButtonPress:

	    if (event.xbutton.window != MyWindows[COLOR].id)
		break;

/*
 *	    Find the row and column indices of the mouse postion where
 *	    the button was pressed
 */
	    i = -999;
	    j = -999;
	    if ((event.xbutton.x - xoff) >= 0)
		i = (event.xbutton.x - xoff) / CMapCellWidth;
	    if ((event.xbutton.y - yoff) >= 0)
		j = (event.xbutton.y - yoff) / CMapCellHeight;
/*
 *	    Calculate the colortable index
 */
	    if ((i < 0) || (i > CMapCols - 1) || (j < 0) || (j > CMapRows - 1))
		bgcolor = 0;
	    else
		bgcolor = (j * CMapCols) + i;
/*
 *	     With a 4-bit visual there are 15 entries The last row/col
 *	    needs to wrap around
 */
	    if (bgcolor > vis->map_entries - 1)
		bgcolor = 0;
/*
 *          Store cyan in the colorcell of the mouse click
 */
	    SavedColor.pixel = bgcolor;
	    SavedColor.flags = DoRed | DoGreen | DoBlue;
	    XQueryColor(display, mycmap, &SavedColor);

	    XStoreNamedColor(display, mycmap, "cyan", bgcolor, clr_mask);
/*
 *          Output colormap id message
 */
	    sprintf(message, "RGB = %6.4f %6.4f %6.4f",
		 (((colors[bgcolor].red / (float) 65535 / image.Scale[0])
		   * (fimage[0].image_max - fimage[0].image_min))
		  + fimage[0].image_min),
	       (((colors[bgcolor].green / (float) 65535 / image.Scale[1])
		 * (fimage[1].image_max - fimage[1].image_min))
		+ fimage[1].image_min),
		(((colors[bgcolor].blue / (float) 65535 / image.Scale[2])
		  * (fimage[2].image_max - fimage[2].image_min))
		 + fimage[2].image_min));

	    OutputMessage(MyWindows[COLOR].id, message);

	    XFlush(display);

	    break;

	case ButtonRelease:

	    if (event.xbutton.window != MyWindows[COLOR].id)
		break;

	    /* Restore the color cell that has been modified. */
	    XStoreColor(display, mycmap, SavedColor);

	    break;

	case KeyPress:

	    XLookupString(&event, string, 20, &key, NULL);
	    keystring = XKeysymToString(key);
	    if (keystring != NULL)
		dispatch_key_event(keystring);

	    break;

	case Expose:

	    if (event.xexpose.window == output_win)
		display_menu(cur_menu);

	    if (event.xexpose.window == graph_win) {
/*
 *		Set the update_picture flag so that when all the events
 *		have been processed we know that the picture needs to be
 *		updated with any changes
 */
		update_picture = TRUE;
		if (!First)
		    XFillRectangle(display, graph_win, gc, 0, 0, gww, gwh);
	    }

	    if (event.xbutton.window == MyWindows[COLOR].id)
		UpdateCMapWin();

	    break;

	case ConfigureNotify:
	    if (event.xconfigure.window == graph_win) {
		gwh = event.xconfigure.width;
		gwh = event.xconfigure.height;
	    }
	    break;

	default:
	    break;

	}	/* end switch() */

    }	/* end while(!exit_program) */

}/* end event_loop() */
