/**
    FitsImageControl.c: Image processing

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Revisions
    01/21/92	BDS	Created.
**/
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>

#include "FitsDefs.h"
#include "FitsOctreeDefs.h"
#include "FitsWindowDefs.h"
#include "FitsMenuDefs.h"
#include "FitsColorDefs.h"
#include "FitsImageDefs.h"

/********************************************************* select_image */
fimage_t  *select_image( FunctionString)
/**
    Select a fits image channel
**/
    char *FunctionString;
{
    fimage_t        ImageId;
    char            Channel;
/*-------------------------------------------------------------------*/
    if (FunctionString[0] != NULL)
	strcpy(CurFunctionString, FunctionString);
    else
	strcpy(CurFunctionString, "Select Image");

    display_menu(SELECTIMAGE_M);
    get_input(&Channel, CHAR);

    CurFunctionString[0] = NULL;

    display_menu(MAIN_M);

    switch (Channel) {
    case 'r':
	return (&fimage[RED]);
	break;
    case 'g':
	return (&fimage[GREEN]);
	break;
    case 'b':
	return (&fimage[BLUE]);
	break;
    default:
	return (NULL);
	break;
    }

}/* end select_image() */

/******************************************************* DeleteImage */
ErrorCode   DeleteImage()
{
    fimage_t       *FImagePtr;
    char            Channel;
/*-------------------------------------------------------------------*/
    FImagePtr = (fimage_t *) select_image("Delete Image");
    if (FImagePtr == NULL)
	return (UNSUCCESSFUL);

    image.channel[FImagePtr->id] = NONE;
    FImagePtr->name[0] = NULL;
    free(FImagePtr->fdata);

}

/************************************************ ToggleImageDisplay */
void	ToggleImageDisplay( )
/**
**/
{
    char            Channel;
    int             i;
    XColor          LColors[256];
/*-------------------------------------------------------------------*/

    display_menu(TOGGLEIMAGE_M);

    get_input(&Channel, CHAR);

    switch (Channel) {
    case 'r':
	if (image.channel[RED] != NONE) {
	    image.channel[RED] = (image.channel[RED] == OFF) ? ON : OFF;
	}
	break;
    case 'g':
	if (image.channel[GREEN] != NONE) {
	    image.channel[GREEN] = (image.channel[GREEN] == OFF) ? ON : OFF;
	}
	break;
    case 'b':
	if (image.channel[BLUE] != NONE) {
	    image.channel[BLUE] = (image.channel[BLUE] == OFF) ? ON : OFF;
	}
	break;
    case 'a':
	if (image.channel[RED] == OFF)
	    image.channel[RED] = ON;
	if (image.channel[GREEN] == OFF)
	    image.channel[GREEN] = ON;
	if (image.channel[BLUE] == OFF)
	    image.channel[BLUE] = ON;
	break;
    default:
	return;
    }

    ColorFlags = 0;
    if (image.channel[RED] == ON)
	ColorFlags |= DoRed;
    if (image.channel[GREEN] == ON)
	ColorFlags |= DoGreen;
    if (image.channel[BLUE] == ON)
	ColorFlags |= DoBlue;

    for (i = 0; i < numcells; i++) {
	LColors[i].pixel = i;
	LColors[i].flags = DoRed | DoGreen | DoBlue;
    }
    XQueryColors(display, mycmap, LColors, numcells);

    for (i = MINCELL; i < numcells; i++) {
	LColors[i].red = (image.channel[RED] == ON) ? colors[i].red : 0;
	LColors[i].green = (image.channel[GREEN] == ON) ? colors[i].green : 0;
	LColors[i].blue = (image.channel[BLUE] == ON) ? colors[i].blue : 0;
    }
    XStoreColors(display, mycmap, LColors, numcells);

    XFlush(display);

    display_menu(MAIN_M);

}/* end ToggleChannelDisplay() */

/************************************************** LoadImagePixelArray */
void LoadImageArray()
/**
    Load the image array with indices into the colormap 
    by finding the image triplet in the octree and using the colormap index
    associated with its branch
**/
{
    int            *IData;
    float          *FData[3], Value[3];
    int             Index;
    int             row, col, i;
/*----------------------------------------------------------------------*/

    IData = image.idata;
    FData[RED] = fimage[RED].fdata;
    FData[GREEN] = fimage[GREEN].fdata;
    FData[BLUE] = fimage[BLUE].fdata;

/*
 *  Load the image array
 *  - any change must also be made to FitsOctreeUtils/OctreeStoreValue()
 */
    for (row = 0; row < image.nrows; row++) {

	for (col = 0; col < image.ncols; col++) {

	    for (i = RED; i <= BLUE; i++) {	/* create color value */
		Value[i] = 0.0;
		if (image.channel[i] != NONE)
		    if ((row < fimage[i].naxis2) && (col < fimage[i].naxis1)) {
			if (*FData[i] < image.RangeMask[i][0])
			    Value[i] = fimage[i].image_min;
			else if (*FData[i] > image.RangeMask[i][1])
			    Value[i] = fimage[i].image_max;
			else
			    Value[i] = *FData[i] * image.Scale[i];
			FData[i]++;
		    }
	    }
/*
 *	    Get the colormap index associated with this color value triplet
 */
	    OctreeGetIndex(RootNode, Value, &Index);
	    *IData++ = Index;

	}

    }

}/* end LoadImageArray() */

/****************************************************** GetNewImage */
ErrorCode GetNewImage( FImagePtr)
/**
    Read in the image and load the currently selected fimage structure
**/
    fimage_t	*FImagePtr;
{
    char            file_name[256];
    FILE           *FitsFilePtr;
    int             i;
    ErrorCode       RC;
/*--------------------------------------------------------------------*/
    display_menu(INPUT_M);
    get_input(file_name, STRING);
    if (file_name[0] == NULL)
	return (UNSUCCESSFUL);

/*
 *  Attempt to open the file.  If not successful then try file_name.fits
 */
    if ((FitsFilePtr = fopen(file_name, "rb")) == NULL) {
	strcat(file_name, ".fits");
	FitsFilePtr = fopen(file_name, "rb");
    }

/*
 *  If we successfully opened the fits data file for read
 *  then load the image in the file into FImagePtr.
 */
    if (FitsFilePtr != NULL) {

	FImagePtr = (fimage_t *) select_image("Add image to which channel");
	if (FImagePtr == NULL)
	    return (UNSUCCESSFUL);

	SetCursor(stopwatch_cursor, ALL_WINDOWS);

	strcpy(FImagePtr->name, file_name);

	RC = read_image_headers(FitsFilePtr, FImagePtr);
	if (RC == SUCCESSFUL)
	    RC = read_image_data(FitsFilePtr, FImagePtr);

	fclose(FitsFilePtr);

	if (RC == SUCCESSFUL) {

	    image.channel[FImagePtr->id] = ON;
	    image.func[FImagePtr->id] = F_NONE;
	    image.RangeMask[FImagePtr->id][0] = 0.0;
	    image.RangeMask[FImagePtr->id][1] = 1.0;
	    image.Scale[FImagePtr->id] = 1.0;

/*
 *	    Calulate the number of rows and columns in the image
 *	    If more memory is required for this new image then allocate it
 */
	    image.nrows = 0;
	    image.ncols = 0;
	    for (i = RED; i <= BLUE; i++) {
		if (image.nrows < fimage[i].naxis1)
		    image.nrows = fimage[i].naxis1;
		if (image.ncols < fimage[i].naxis2)
		    image.ncols = fimage[i].naxis2;
	    }
	    image.NumPixels = image.ncols * image.nrows;
	    if (sizeof(image.idata) < (image.ncols * image.nrows)) {
		free(image.idata);
		image.idata = (int *) malloc(sizeof(long) * image.NumPixels);
	    }

	    RC = SUCCESSFUL;

	}

    }
    else {
	
	ReportError(READ_ERROR, file_name);
	RC = READ_ERROR;

    }

    SetCursor(normal_cursor, ALL_WINDOWS);
    return (RC);

}/* end GetNewImage() */

/********************************************** ImageControl */
ErrorCode   ImageControl()
{
    char            CharCode;
    int             i;
    ErrorCode       RC = SUCCESSFUL;
/*-----------------------------------------------------------*/
    display_menu(IMAGECONTROL_M);
    get_input(&CharCode, CHAR);

    switch (CharCode) {
    case 'i':	/* Read in a new FITS image */
	if (GetNewImage() == SUCCESSFUL)
	    RegenImage();
	break;
    case 'd':	/* Display the image */
	for (i = RED; i < BLUE; i++)
	    if (image.channel[i] != NONE) {
		DeleteImage();
		break;
	    }
	break;
    case 's':
	for (i = RED; i < BLUE; i++)
	    if (image.channel[i] != NONE) {
		ScaleImage();
		break;
	    }
	break;
    case 't':
	for (i = RED; i < BLUE; i++)
	    if (image.channel[i] != NONE) {
		ToggleImageDisplay();
		break;
	    }
	break;
    }

    display_menu(MAIN_M);

}
