/**
    FitsImageProcessing.c - Manipulate the image in whatever way 
                            be it apllying functions to scaling

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Author Change Log
    01/21/92	BDS	Created.
    02/10/92	BDS	Added scaling (range focus))
**/

#include <stdio.h>
#include <math.h>

#include "FitsMenuDefs.h"
#include "FitsDefs.h"
#include "FitsOctreeDefs.h"
#include "FitsColorDefs.h"
#include "FitsImageDefs.h"

/****************************************************** SetFunction */
void	SetFunction( )
/**
**/
{
    char            Func;
    int             Channel;
    int             i;
    fimage_t       *FImagePtr;
    double          OldColor;
    double          Range;
/*-------------------------------------------------------------------*/
    display_menu(FUNC_M);
    get_input(&Func, CHAR);

    FImagePtr = (fimage_t *) select_image("Change function of:");
    if ((FImagePtr == NULL) || (image.channel[FImagePtr->id] != ON))
	return;
    Channel = FImagePtr->id;

    switch (Func) {

    case 's':	/* square */
	for (i = MINCELL; i <= image.MaxColorIndex; i++) {
	    switch (Channel) {
	    case RED:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].red;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].red = (OldColor * OldColor) * 65535;
		break;
	    case GREEN:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].green;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].green = (OldColor * OldColor) * 65535;
		break;
	    case BLUE:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].blue;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].blue = (OldColor * OldColor) * 65535;
		break;
	    }
	}
	image.func[Channel] = F_SQR;
	break;

    case 'r':	/* square root */
	for (i = MINCELL; i <= image.MaxColorIndex; i++) {
	    switch (Channel) {
	    case RED:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].red;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].red = sqrt(OldColor) * 65535;
		break;
	    case GREEN:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].green;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].green = sqrt(OldColor) * 65535;
		break;
	    case BLUE:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].blue;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		colors[i].blue = sqrt(OldColor) * 65535;
		break;
	    }
	}
	image.func[Channel] = F_SQRT;
	break;

    case 'l':	/* log */
	for (i = MINCELL; i <= image.MaxColorIndex; i++) {
	    switch (Channel) {
	    case RED:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].red;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		Range = fimage[Channel].image_max - fimage[Channel].image_min;
		OldColor = OldColor * Range;
		colors[i].red = log(OldColor + 1) / log(Range + 1) * 65535;
		break;
	    case GREEN:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].green;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		Range = fimage[Channel].image_max - fimage[Channel].image_min;
		OldColor = OldColor * Range;
		colors[i].green = log(OldColor + 1) / log(Range + 1) * 65535;
		break;
	    case BLUE:
		if (image.func[Channel] == F_NONE)
		    SavedColors[i][Channel] = colors[i].blue;
		OldColor = (double) SavedColors[i][Channel] / (double) 65535;
		Range = fimage[Channel].image_max - fimage[Channel].image_min;
		OldColor = OldColor * Range;
		colors[i].blue = log(OldColor + 1) / log(Range + 1) * 65535;
		break;
	    }
	}
	image.func[Channel] = F_LOG;
	break;

    case 'n':	/* none */
	if (image.func[Channel] != F_NONE) {
	    for (i = MINCELL; i <= image.MaxColorIndex; i++) {
		switch (Channel) {
		case RED:
		    colors[i].red = SavedColors[i][Channel];
		    break;
		case GREEN:
		    colors[i].green = SavedColors[i][Channel];
		    break;
		case BLUE:
		    colors[i].blue = SavedColors[i][Channel];
		    break;
		}
	    }
	    image.func[Channel] = F_NONE;
	}
	break;

    default:

	return;
    }

    XStoreColors(display, mycmap, colors, numcells);
    XInstallColormap(display, mycmap);

    XFlush(display);

    display_menu(MAIN_M);

}/* end SetFunction() */

/******************************************************* ScaleImage */
void ScaleImage( )
/**
    Scale the image by asking the user for a new min/max value pair
**/
{
    fimage_t       *FImagePtr;
    float           MinVal = 0, MaxVal = 0;
    Logical         Regen = FALSE;
    static float    LastMinVal[3], LastMaxVal[3];
    int             ImageId;
/*------------------------------------------------------------------*/

    FImagePtr = (fimage_t *) select_image("Select Image To Scale");
    if ((FImagePtr == NULL) || (image.channel[FImagePtr->id] != ON))
	return;

    MinVal = FImagePtr->image_min;
    MaxVal = FImagePtr->image_max;

/*
 *  Get the new min and max values from the user
 */
    menu[SCALE_M].num_lines = 6;
    sprintf(menu[SCALE_M].text[0], "Image: %s", FImagePtr->name);
    sprintf(menu[SCALE_M].text[1], " Full Range = %f to %f",
	    FImagePtr->data_min, FImagePtr->data_max);
    sprintf(menu[SCALE_M].text[2], "  Min = %f", FImagePtr->image_min);
    sprintf(menu[SCALE_M].text[3], "  Max = %f", FImagePtr->image_max);
    strcpy(menu[SCALE_M].text[4], "");
    strcpy(menu[SCALE_M].text[5], "Enter new MIN:");

    display_menu(SCALE_M);
    get_input(&MinVal, FLOAT);

    if (MinVal < FImagePtr->data_min)
	MinVal = FImagePtr->data_min;
    if (MinVal != FImagePtr->image_min) {
	Regen = TRUE;
	FImagePtr->image_min = MinVal;
	sprintf(menu[SCALE_M].text[2], "  Min = %f", FImagePtr->image_min);
    }

    strcpy(menu[SCALE_M].text[5], "Enter new MAX:");

    display_menu(SCALE_M);
    get_input(&MaxVal, FLOAT);
/*
 *  if MaxVal is greater than fimage max than set to max
 */
    if (MaxVal > FImagePtr->data_max)
	MaxVal = FImagePtr->data_max;

    if (MaxVal != FImagePtr->image_max) {
	FImagePtr->image_max = MaxVal;
	Regen = TRUE;
    }

    if (Regen == TRUE) {
	image.RangeMask[FImagePtr->id][0] =
	    (MinVal - FImagePtr->data_min) / (FImagePtr->data_max);
	image.RangeMask[FImagePtr->id][1] =
	    (MaxVal - FImagePtr->data_min) /
	    (FImagePtr->data_max - FImagePtr->data_min);
	image.Scale[FImagePtr->id] =
	    1 /
	    (image.RangeMask[FImagePtr->id][1]
	     - image.RangeMask[FImagePtr->id][0]);
	RegenImage();
    }

    display_menu(MAIN_M);

}/* end ScaleImage() */
