/**
    FitsInit.c: Initialize everything

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Author Change Log
    01/20/92	BDS     Created.
    03/03/92	BDS	Init fimage[].bscale to 1.0
**/

#include <stdio.h>

#define  MainStore
#include "FitsDefs.h"
#include "FitsX11Defs.h"
#include "FitsColorDefs.h"
#include "FitsImageDefs.h"
#include "FitsOctreeDefs.h"
#include "FitsWindowDefs.h"
#include "FitsMenuDefs.h"
#include "FitsIODefs.h"

/******************************************************* get_visual() */
Bool get_visual(depth)
/**
    Get a visual of depth *depth if possible
**/
    int    *depth;  /* The depth of the desired visual */
{
    XVisualInfo *visualList;
    long        vmask = VisualNoMask;
    XVisualInfo vinfo, *v;           /* visual information */
    int         big_map = 0, istat;
    Bool        success = FALSE;
/*--------------------------------------------------------------------*/
    visualList = XGetVisualInfo( display, vmask, &vinfo, &istat);

    switch (*depth) {
        case 4:
            for (v = visualList; v < visualList + istat; v++)
                if ((v->depth == 4)&&(v->class == PseudoColor)) {
                    vinfo = *v;
                    success = TRUE;
                    break;
                }
            break;
        case 8:
            for (v = visualList; v < visualList + istat; v++)
                if ((v->depth == 8)&&(v->class == PseudoColor)) {
                    vinfo = *v;
                    success = TRUE;
                    break;
                }
            break;
        default:
            for (v = visualList; v < visualList + istat; v++) {
                if ((v->colormap_size>big_map)&&(v->class==PseudoColor)) {
                    big_map = v->colormap_size;
                    vinfo   = *v;
                    success = TRUE;
                }
                if (big_map >= 256) break;
            }
    }; /* end switch(depth) */

    if (success) {
        *depth   = vinfo.depth;
        vis      = vinfo.visual;
        numcells = vis->map_entries;
        return(TRUE);
    }
    else
        return(FALSE);

} /* end get_visual() */

/**************************************************** init_Xdisplay */
init_Xdisplay()
/**
    Open a connection to the XServer,
    Init some display type variable and
    Create the main display window
**/
{
    int             num;
    int             index;
/*--------------------------------------------------------------------*/
    if (display)
	return;

    /* Initialize the connection to the XServer and get some default
     * values for a start */
    display = XOpenDisplay(NULL);
    screen = XDefaultScreen(display);
    dcm = DefaultColormap(display, screen);
    vis = DefaultVisual(display, screen);
    root = RootWindow(display, screen);
    dcms = vis->map_entries;
    dep = DefaultDepth(display, screen);
    truecolor = (vis->class == TrueColor) ? TRUE : FALSE;

    dep = 8;
    get_visual(&dep);
    c_range = numcells - MINCELL;

/*
 *  Initialize the colors array's pixels value
 *  for use in the QueryColors call.  The pixel field needs to be set
 *  to the colortable index of the color that we want information about
 */
    ColorFlags = DoRed | DoGreen | DoBlue;
    for (index = 0; index < numcells; index++) {
	colors[index].pixel = index;
	colors[index].flags = ColorFlags;
    }

/*
 *  Initialize my colormap
 */
    mycmap = XCreateColormap(display, root, vis, AllocAll);
    icm = XListInstalledColormaps(display, root, &num);
    XQueryColors(display, *icm, colors, dcms);
    XStoreColors(display, mycmap, colors, dcms);

/*
 *  Create the cursors 
 */
    stopwatch_cursor = XCreateFontCursor(display, XC_watch);
    normal_cursor = XCreateFontCursor(display, XC_arrow);

}/* end init_Xdisplay() */

/*************************************************************** main */
void FitsInit( )
{
    int             i;
/*--------------------------------------------------------------------*/
    fimage_ptr = NULL;
/*
 *  init the three fits image control structures
 */
    for (i = RED; i <= BLUE; i++) {
	fimage[i].name[0] = '\0';
	fimage[i].id = i;
	fimage[i].bitpix = 0;
	fimage[i].naxis1 = 0;
	fimage[i].naxis2 = 0;
	fimage[i].bscale = 1.0;
	fimage[i].bzero = 0.0;
	fimage[i].data_max = 0;
	fimage[i].data_min = 0;
	fimage[i].fdata = NULL;
    }

/*
 *  init the display image control structure
 */
    image.nrows = 0;
    image.ncols = 0;

    for (i = RED; i <= BLUE; i++) {
	image.channel[i] = NONE;
	image.RangeMask[i][0] = 0.0;
	image.RangeMask[i][1] = 1.0;
	image.Scale[i] = 1;
	image.func[i] = F_NONE;
    }

    image.idata = NULL;

/*
 *  init the window and text size globbies
 */
    graph_win = 0;
    ImageCellWidth = 6;
    ImageCellHeight = 6;
    text_width = 8;
    text_height = 12;
    CurFunctionString[0] = '/0';
    exit_program = FALSE;

/*
 *  Create the octree that will hold the composite image
 */
    RootNode = (struct OctreeNode *) OctreeCreateRoot();

/*
 *  int stuff for the menus
 */
    menu = (menu_t *) malloc(NUM_MENUS * sizeof(menu_t));
    cur_menu = MAIN_M;

/*
 *  Init stuff for FitsIO
 */
    simple = '\0';
    naxis = -1;

/*
 *  Init the X display
 */
    init_Xdisplay();

}/* end FitsInit() */
