/***
    FitsOctreeUtils.c - Octree Utility Routines

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Auther Change Log
    01/20/92	BDS	Created.
***/

#include "FitsDefs.h"
#include "FitsImageDefs.h"
#include "FitsOctreeDefs.h"

#include <stdio.h>
#include <stdlib.h>

/******************************************************** OctreePrint */
void  OctreePrint( NodePtr)
    struct OctreeNode *NodePtr;
/**
    Print the values for NodePtr and its children
**/
{
    int             c, i, NodeId;
    struct OctreeNode *Node2Ptr;
/*--------------------------------------------------------------------*/
    if (NodePtr->Id == -1) {
	for (i = 0; i < OCTREE_MAX_NODE_DEPTH; i++) {
	    printf(" %d/", i % 10);
	}
	printf("\n--------------------------------------------------------\n");
    }

    for (c = 0; c < 8; c++) {
	if (NodePtr->Child[c] != NULL) {
	    OctreePrint(NodePtr->Child[c]);
	}
    }

    if ((NodePtr->Value[0] != -1) && (NodePtr->Id != -1)) {

	Node2Ptr = RootNode;
	for (i = NodePtr->Depth; i > 0; i--) {
	    NodeId = 0;
	    if (NodePtr->Value[0] >=
		((Node2Ptr->MaxVal[0] + Node2Ptr->MinVal[0]) / 2))
		NodeId += 1;
	    if (NodePtr->Value[1] >=
		((Node2Ptr->MaxVal[1] + Node2Ptr->MinVal[1]) / 2))
		NodeId += 2;
	    if (NodePtr->Value[2] >=
		((Node2Ptr->MaxVal[2] + Node2Ptr->MinVal[2]) / 2))
		NodeId += 4;

	    printf("%d/ ", Node2Ptr->Id);
	    Node2Ptr = Node2Ptr->Child[NodeId];

	}

	printf("%d = %f %f %f, cid=%d, nv=%d\n", NodePtr->Id,
	       NodePtr->Value[0], NodePtr->Value[1], NodePtr->Value[2],
	       NodePtr->Index, NodePtr->NumVals);

    }
    else {
	printf("\nDepth -----------> ");
	for (i = 0; i < OCTREE_MAX_NODE_DEPTH; i++) {
	    printf("%d", i % 10);
	}
	printf("\n-------------------------------------");
	printf("\nLeavesPerLevel   = ");
	for (i = 0; i < OCTREE_MAX_NODE_DEPTH; i++) {
	    printf("%d", LeavesPerLevel[i]);
	}
	printf("\nChildrenPerLevel = ");
	for (i = 0; i < OCTREE_MAX_NODE_DEPTH; i++) {
	    printf("%d", ChildrenPerLevel[i]);
	} printf("\n");

    }

}

/******************************************************* OctreeNodeCreate */
int OctreeNodeCreate( Parent, NodeId)
    struct  OctreeNode *Parent;
    int NodeId;
/**
    Create a new child octree node NodeId for Parent.
**/
{
    struct OctreeNode *NewNode;
    int         i;
    float      *PMax, *PMin;
/*------------------------------------------------------------------------*/
/*
 *  Alloc memory for the new child node
 */
    NewNode = (struct OctreeNode *) malloc(sizeof(struct OctreeNode));

/*
 *  if we failed to get the memory then return.
 */
    if (NewNode == NULL)
	return (OCTREE_MALLOC_FAILED);

/*
 *  Init the control vars in the new child
 */
    NewNode->Id = NodeId;
    NewNode->Depth = Parent->Depth + 1;
    NewNode->NumVals = 0;
    NewNode->Index = 0;
    NewNode->Value[0] = 0;
    NewNode->Value[1] = 0;
    NewNode->Value[2] = 0;

    PMax = Parent->MaxVal;
    PMin = Parent->MinVal;

/*
 *  Set the minimum and maximum values
 *  depending on the parents bounding box
 */
    switch (NodeId) {
	case 0:
	    NewNode->MinVal[0] = Parent->MinVal[0];
	    NewNode->MinVal[1] = Parent->MinVal[1];
	    NewNode->MinVal[2] = Parent->MinVal[2];
	    NewNode->MaxVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MaxVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MaxVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    break;
	case 1:
	    NewNode->MinVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MinVal[1] = Parent->MinVal[1];
	    NewNode->MinVal[2] = Parent->MinVal[2];
	    NewNode->MaxVal[0] = Parent->MaxVal[0];
	    NewNode->MaxVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MaxVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    break;
	case 2:
	    NewNode->MinVal[0] = Parent->MinVal[0];
	    NewNode->MinVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MinVal[2] = Parent->MinVal[2];
	    NewNode->MaxVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MaxVal[1] = Parent->MaxVal[1];
	    NewNode->MaxVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    break;
	case 3:
	    NewNode->MinVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MinVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MinVal[2] = Parent->MinVal[2];
	    NewNode->MaxVal[0] = Parent->MaxVal[0];
	    NewNode->MaxVal[1] = Parent->MaxVal[1];
	    NewNode->MaxVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    break;
	case 4:
	    NewNode->MinVal[0] = Parent->MinVal[0];
	    NewNode->MinVal[1] = Parent->MinVal[1];
	    NewNode->MinVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    NewNode->MaxVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MaxVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MaxVal[2] = Parent->MaxVal[2];
	    break;
	case 5:
	    NewNode->MinVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MinVal[1] = Parent->MinVal[1];
	    NewNode->MinVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    NewNode->MaxVal[0] = Parent->MaxVal[0];
	    NewNode->MaxVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MaxVal[2] = Parent->MaxVal[2];
	    break;
	case 6:
	    NewNode->MinVal[0] = Parent->MinVal[0];
	    NewNode->MinVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MinVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    NewNode->MaxVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MaxVal[1] = Parent->MaxVal[1];
	    NewNode->MaxVal[2] = Parent->MaxVal[2];
	    break;
	case 7:
	    NewNode->MinVal[0] = (Parent->MaxVal[0] + Parent->MinVal[0]) / 2;
	    NewNode->MinVal[1] = (Parent->MaxVal[1] + Parent->MinVal[1]) / 2;
	    NewNode->MinVal[2] = (Parent->MaxVal[2] + Parent->MinVal[2]) / 2;
	    NewNode->MaxVal[0] = Parent->MaxVal[0];
	    NewNode->MaxVal[1] = Parent->MaxVal[1];
	    NewNode->MaxVal[2] = Parent->MaxVal[2];
	    break;
    }

/*
 *  Init all the Child's children to NULL
 */
    for (i = 0; i < 8; i++) {
	NewNode->Child[i] = NULL;
    }

/*
 *  Link the child to the parent
 */
    NewNode->Parent = Parent;
    Parent->Child[NodeId] = NewNode;

    return (SUCCESSFUL);

}				/* end OctreeNodeCreate() */

/******************************************************* OctreeNodeDelete */
int   OctreeNodeDelete( NodePtr)
    struct OctreeNode *NodePtr;
/**
    Free all the memory associated with the octree node refd by NodePtr
**/
{
    int         c;
/*------------------------------------------------------------------------*/
/*
 *  Don't delete that which doesn't exist
 */
    if (NodePtr == NULL)
	return(-1);

/*
 *  Clear and free the memory of NodePtr's child nodes
 */
    for (c = 0; c < 8; c++) {
	if (NodePtr->Child[c] != NULL) {
	    OctreeNodeDelete(NodePtr->Child[c]);
	}
    }

/*
 *  if this is not the root node 
 *  then Set the parents pointer to this node to NULL
 *  and Free the memory associated with this node
 */
    if (NodePtr->Id != OCTREE_ROOT_NODE_ID) {
	NodePtr->Parent->Child[NodePtr->Id] = NULL;
	free(NodePtr);
    }

    return (0);

}				/* end OctreeNodeDelete() */

/******************************************************* OctreeCreateRoot */
struct OctreeNode *OctreeCreateRoot()
/**
    Create a root node of an octree
**/
{
    struct OctreeNode *RootNodePtr;
    int         c;
/*------------------------------------------------------------------------*/
/*
 *  Create the root node
 */
    RootNodePtr = (struct OctreeNode *) malloc(sizeof(struct OctreeNode));
    if (RootNodePtr == NULL)
	return (NULL);

/*
 *  Init the root control variables
 */
    RootNodePtr->Id = OCTREE_ROOT_NODE_ID;
    RootNodePtr->Depth = 0;
    RootNodePtr->MinVal[0] = 0;
    RootNodePtr->MinVal[1] = 0;
    RootNodePtr->MinVal[2] = 0;
    RootNodePtr->MaxVal[0] = 1.0;
    RootNodePtr->MaxVal[1] = 1.0;
    RootNodePtr->MaxVal[2] = 1.0;
    RootNodePtr->Value[0] = -1;
    RootNodePtr->Value[1] = -1;
    RootNodePtr->Value[2] = -1;
    RootNodePtr->NumVals = 0;
    RootNodePtr->Parent = NULL;

/*
 *  Init all children to NULL
 */
    for (c = 0; c < 8; c++) {
	RootNodePtr->Child[c] = NULL;
    }

/*
 *  Return a pointer to the root node
 */
    return (RootNodePtr);

}				/* end OctreeCreateRoot() */

/*********************************************************** OctreeReset */
int OctreeReset( RootNodePtr)
    struct OctreeNode *RootNodePtr;
/**
    Reset the octree by deleting all descendents of the root octant
**/
{
    int             i;
/*------------------------------------------------------------------------*/
/*
 *  Reset the root control variables
 */
    RootNodePtr->Id = OCTREE_ROOT_NODE_ID;
    RootNodePtr->Depth = 0;
    RootNodePtr->Index = 0;
    RootNodePtr->Value[RED] = -1;
    RootNodePtr->Value[GREEN] = -1;
    RootNodePtr->Value[BLUE] = -1;
    RootNodePtr->NumVals = 0;
    RootNodePtr->Parent = NULL;

/*
 *  Delete all the living children
 */
    for (i = 0; i < 8; i++) {
	if (RootNodePtr->Child[i] != NULL)
	    OctreeNodeDelete(RootNodePtr->Child[i]);
    }

/*
 *  Reset the LeavesPerLevel and ChildrenPerLevel counts
 */
    for (i = 0; i < OCTREE_MAX_NODE_DEPTH; i++) {
	LeavesPerLevel[i] = 0;
	ChildrenPerLevel[i] = 0;
    }

}/* end OctreeReset() */

/**************************************************** OctreeStoreValue */
void OctreeStoreValue( NodePtr, NewValue)
    struct OctreeNode *NodePtr;
    float               NewValue[3];
/**
    Find the octant where NewValue should go and store NewValue there
**/
{
    int             c = 0;
    int             NodeId = 0;
    float           TmpVal[3];
    int             NewNumVals;
    Logical         IsLeaf = TRUE;
/*--------------------------------------------------------------------*/
/*
 *  If we are at the max depth then just increment
 *  else continue placement
 */
    if (NodePtr->Depth >= OCTREE_MAX_NODE_DEPTH) {
	NodePtr->NumVals++;
	return;
    }

    for (c = 0; c < 8; c++) {
	if (NodePtr->Child[c] != NULL) {
	    IsLeaf == FALSE;
	}
    }

/*
 *  If this node only has one value associated with it
 *  and the new value is identical to the value stored here
 *  then we're done
 */
    if (IsLeaf) {
	if (abs(NewValue[0] - NodePtr->Value[0]) < TOL)
	    if (abs(NewValue[1] - NodePtr->Value[1]) < TOL)
		if (abs(NewValue[2] - NodePtr->Value[2]) < TOL) {
		    NodePtr->NumVals++;
		    return;
		}
    }

/*
 *  Calc which octant to put this new value - build by bits
 */
    if (NewValue[0] >= ((NodePtr->MaxVal[0] + NodePtr->MinVal[0]) / 2))
	NodeId += 1;
    if (NewValue[1] >= ((NodePtr->MaxVal[1] + NodePtr->MinVal[1]) / 2))
	NodeId += 2;
    if (NewValue[2] >= ((NodePtr->MaxVal[2] + NodePtr->MinVal[2]) / 2))
	NodeId += 4;

/*
 *  If the child doesn't exist then create it and load it
 *  else continue our placement search
 */
    if (NodePtr->Child[NodeId] == NULL) {

	OctreeNodeCreate(NodePtr, NodeId);
	NodePtr->Child[NodeId]->Value[0] = NewValue[0];
	NodePtr->Child[NodeId]->Value[1] = NewValue[1];
	NodePtr->Child[NodeId]->Value[2] = NewValue[2];
	NodePtr->Child[NodeId]->NumVals = 1;
/*
 *      if the NumVals in this node was 1 then we need to move
 *      the value stored here down
 */
	if (IsLeaf) {
	    TmpVal[0] = NodePtr->Value[0];
	    TmpVal[1] = NodePtr->Value[1];
	    TmpVal[2] = NodePtr->Value[2];
	    NodePtr->Value[0] = -1;
	    NodePtr->Value[1] = -1;
	    NodePtr->Value[2] = -1;
	    OctreeStoreValue(NodePtr, TmpVal);
	}

    }
    else {

	OctreeStoreValue(NodePtr->Child[NodeId], NewValue);
	NodePtr->NumVals++;

    }

}/* OctreeStoreValue() */

/***************************************************** OctreeGetIndex */
void   OctreeGetIndex( NodePtr, Value, Index)
/**
    Search the tree for Value and return the colormap index of the branch/leaf
**/
    struct OctreeNode *NodePtr;
    float	      Value[];
    int		      *Index;
{
    int         c = 0;
    int         NodeId = 0;
/*--------------------------------------------------------------------*/
/*
 *  If we are at the max depth then set index to this node`s index
 */
    if (NodePtr->Depth >= ColorLevelMax) {
	*Index = NodePtr->Index;
	return;
    }

/*
 *  Calc which octant to search further
 */
    if (Value[0] >= ((NodePtr->MaxVal[0]+NodePtr->MinVal[0]) / 2))
	NodeId += 1;
    if (Value[1] >= ((NodePtr->MaxVal[1]+NodePtr->MinVal[1]) / 2))
	NodeId += 2;
    if (Value[2] >= ((NodePtr->MaxVal[2]+NodePtr->MinVal[2]) / 2))
	NodeId += 4;

/*
 *  If the target octant doesn't exist
 *  then set Index to this node's index
 *  else continue our search
 */
    if (NodePtr->Child[NodeId] == NULL)
	*Index = NodePtr->Index;
    else
	OctreeGetIndex(NodePtr->Child[NodeId], Value, Index);

}				/* OctreeGetIndex() */

/***************************************************** OctreeCalcStats */
ErrorCode OctreeCalcStats( NodePtr)
/**
    Calculate the number of Children per depth level 
    and the number of Leaves per level
**/
    struct OctreeNode	*NodePtr;
{
    int             i, ChildCount = 0;
/*---------------------------------------------------------------------*/

    for (i = 0; i < 8; i++) {
	if (NodePtr->Child[i] != NULL) {
	    OctreeCalcStats(NodePtr->Child[i]);
	    ChildCount++;
	}
    }

    if (ChildCount == 0) {	/* this is a leaf node */
	LeavesPerLevel[NodePtr->Depth]++;
    }
    ChildrenPerLevel[NodePtr->Depth]++;

    return (SUCCESSFUL);

}/* end OctreeCalcStats() */

/********************************************************* LoadOctree */
void LoadOctree()
/**
    Take any/all three images and load the composit into the octree.
**/
{
    float          *FData[3], Value[3];
    int             Index;
    int             row, col, i;
/*----------------------------------------------------------------------*/

    OctreeReset(RootNode);

    FData[RED] = fimage[RED].fdata;
    FData[GREEN] = fimage[GREEN].fdata;
    FData[BLUE] = fimage[BLUE].fdata;

    for (row = 0; row < image.nrows; row++) {

	for (col = 0; col < image.ncols; col++) {
/*
 *	    Create Color Value Triplet
 *	    - any change must also be made to FitsImageControl/LoadImageArray()
 */
	    for (i = RED; i <= BLUE; i++) {
		Value[i] = 0.0;
		if (image.channel[i] != NONE)
		    if ((row < fimage[i].naxis2) && (col < fimage[i].naxis1)) {
			if (*FData[i] < image.RangeMask[i][0])
			    Value[i] = fimage[i].image_min;
			else if (*FData[i] > image.RangeMask[i][1])
			    Value[i] = fimage[i].image_max;
			else
			    Value[i] = *FData[i] * image.Scale[i];
			FData[i]++;
		    }
	    }
/*
 *	    Load the triplet into our octree
 */
	    OctreeStoreValue(RootNode, Value);

	}

    }

}/* end LoadOctree() */
