/**
    FitsUIControl.c: User Interface and Menu handling

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Revisions
    01/21/92	BDS	Created.
**/

#include <stdio.h>

#include "FitsDefs.h"
#include "FitsImageDefs.h"
#include "FitsColorDefs.h"
#include "FitsWindowDefs.h"
#include "FitsMenuDefs.h"

/***************************************************** OutputMessage */
void OutputMessage(win, message)
/**
    Output text to the output window
**/
    Window win;
    char   message[100];
{
    int             mlen;
/*--------------------------------------------------------------------*/
    if (win == MyWindows[COLOR].id) {
	XSetForeground(display, gc, 0);
	XFillRectangle(display, win, gc, 0, 0,
		       MyWindows[COLOR].w, banner_height);
	mlen = strlen(message);
	XDrawImageString(display, win, gc, 10, 15, message, strlen(message));
	XFlush(display);
    }
    else {
	XSetForeground(display, gc, 0);
	XFillRectangle(display, win, gc, 10, 19, oww, owh);
	mlen = strlen(message);
	XDrawImageString(display, win, gc, 10, 30, message, mlen);
	XFlush(display);
    }
}

/******************************************************* create_menus */
void create_menus( )
{
/*--------------------------------------------------------------------*/
    MenuGC = XCreateGC(display, output_win, 0, 0);

    menu[MAIN_M].num_lines = 7;
    strcpy(menu[MAIN_M].text[0], "Choose a function");
    strcpy(menu[MAIN_M].text[1], "  A - Animate Colormap           ");
    strcpy(menu[MAIN_M].text[2], "  F - Change transFormation...   ");
    strcpy(menu[MAIN_M].text[3], "  I - Image Control...           ");
    strcpy(menu[MAIN_M].text[4], "  D - Display Control...         ");
    strcpy(menu[MAIN_M].text[5], "  C - Load Predefd Colormap...   ");

    menu[INPUT_M].num_lines = 1;
    strcpy(menu[INPUT_M].text[0], "Enter the name of the image");

    menu[FUNC_M].num_lines = 5;
    strcpy(menu[FUNC_M].text[0], "Choose Transformation:");
    strcpy(menu[FUNC_M].text[1], "  N - none        ");
    strcpy(menu[FUNC_M].text[2], "  L - log         ");
    strcpy(menu[FUNC_M].text[3], "  S - square      ");
    strcpy(menu[FUNC_M].text[4], "  R - square root ");

    menu[COLOR_M].num_lines = 7;
    strcpy(menu[COLOR_M].text[0], "What type of colormap to load?");
    strcpy(menu[COLOR_M].text[1], " R - load RED       color ramp");
    strcpy(menu[COLOR_M].text[2], " G - load GREEN     color ramp");
    strcpy(menu[COLOR_M].text[3], " B - load BLUE      color ramp");
    strcpy(menu[COLOR_M].text[4], " S - load GREYSCALE color ramp");
    strcpy(menu[COLOR_M].text[5], " D - load DEFAULT   color ramp");
    strcpy(menu[COLOR_M].text[6], " M - load MIXED     color ramp");

    menu[ANIMATE_M].num_lines = 1;
    strcpy(menu[ANIMATE_M].text[0], "Set Cycle delay in Milliseconds");

    menu[IMAGECONTROL_M].num_lines = 5; /* see ImageControl() */
    strcpy(menu[IMAGECONTROL_M].text[0], "Enter choice:");
    strcpy(menu[IMAGECONTROL_M].text[1], "  I - Input new Image Component ");
    strcpy(menu[IMAGECONTROL_M].text[2], "  D - Delete Image Component    ");
    strcpy(menu[IMAGECONTROL_M].text[3], "  S - Scale  Image Component    ");
    strcpy(menu[IMAGECONTROL_M].text[4], "  T - Toggle Image Component    ");

    menu[DISPCONTROL_M].num_lines = 3; /* see DisplayControl() */
    strcpy(menu[DISPCONTROL_M].text[0], "Enter choice:");
    strcpy(menu[DISPCONTROL_M].text[1], "  R - reGenerate Image ");
    strcpy(menu[DISPCONTROL_M].text[2], "  D - Refresh Image    ");


}/* end create_menus() */

/******************************************************* display_menu */
void display_menu( menu_id)
/**
    Display the menu specified by menu_id
**/
    int menu_id;
{
    int             mlen, line, i;
    char            buffer[40];
    char            color[3];
/*--------------------------------------------------------------------*/
    color[0] = 'R';
    color[1] = 'G';
    color[2] = 'B';

    cur_menu = menu_id;	/* Store menu_id as the current menu displayed */

    XClearWindow(display, output_win);
    XSync(display, 0);

    switch (menu_id) {

    case SELECTIMAGE_M:

	menu[SELECTIMAGE_M].num_lines = 4;
	strcpy(menu[SELECTIMAGE_M].text[0], CurFunctionString);
	for (i = RED; i <= BLUE; i++) {
	    buffer[0] = '\0';
	    switch (image.channel[i]) {
	    case ON:
		sprintf(buffer, "%c (ON ) - %s",
			color[i], fimage[i].name);
		break;
	    case OFF:
		sprintf(buffer, "%c (OFF) - %s",
			color[i], fimage[i].name);
		break;
	    case NONE:
		sprintf(buffer, "%c (UND) - %s",
			color[i], fimage[i].name);
		break;
	    }
	    strcpy(menu[SELECTIMAGE_M].text[i + 1], buffer);
	}

	break;

    case TOGGLEIMAGE_M:

	menu[TOGGLEIMAGE_M].num_lines = 5;
	strcpy(menu[TOGGLEIMAGE_M].text[0], "Toggle Image Display");
	for (i = RED; i <= BLUE; i++) {
	    buffer[0] = '\0';
	    switch (image.channel[i]) {
	    case ON:
		sprintf(buffer, "%c (ON ) - %s", color[i], fimage[i].name);
		break;
	    case OFF:
		sprintf(buffer, "%c (OFF) - %s", color[i], fimage[i].name);
		break;
	    case NONE:
		sprintf(buffer, "%c (UND) - %s", color[i], fimage[i].name);
		break;
	    }
	    strcpy(menu[TOGGLEIMAGE_M].text[i + 1], buffer);
	}
	strcpy(menu[TOGGLEIMAGE_M].text[i + 1], "A - Turn all on");

	break;

    default:
	break;

    }


    /* Display the new menu */
    for (line = 0; line < menu[menu_id].num_lines; line++) {
	mlen = strlen(menu[menu_id].text[line]);
	XDrawImageString(display, output_win, MenuGC,
			 10, 15 + text_height * line,
			 menu[menu_id].text[line], mlen);
    }

    XFlush(display);

}/* end display_menu() */

/************************************************** create_ui */
create_ui()
/**
    Create the user interface windows and map
**/
{
/*------------------------------------------------------------*/
    /* Calculate the size and postion of the output window */
    owx1 = 20;
    owy1 = 20;
    oww = 30 * text_width;
    owh = 12 * text_height;
    owx2 = owx1 + oww;
    owy2 = owy1 + owh;

    output_win = create_window(root, owx1, owy1, oww, owh, 1,
			       "FitsImaging v2.1");

}/* end create_ui() */

/****************************************************** CursorSet */
void SetCursor(CursorStyle, WindowId)
/**
    Change the cursor for the specified window
**/
    int	CursorStyle;
    int	WindowId;
{
/*----------------------------------------------------------------*/
    if (WindowId == ALL_WINDOWS) {
	if (output_win != NULL)
	    XDefineCursor(display, output_win, CursorStyle);
	if (graph_win != NULL)
	    XDefineCursor(display, graph_win, CursorStyle);
	if (MyWindows[COLOR].id != NULL)
	    XDefineCursor(display, MyWindows[COLOR].id, CursorStyle);
	
    }
    else if (WindowId != NULL)
	XDefineCursor(display, WindowId, CursorStyle);

    XFlush(display);
}

/**************************************************** ReportError */
void	ReportError( ErrorNumber, Arg1)
/**
**/
    int	ErrorNumber;
    char    *Arg1;
{
    char            message[1024];
    int             mlen;
/*----------------------------------------------------------------*/
    switch (ErrorNumber) {
    case READ_ERROR:
	sprintf(message, "Error opening \"%s\" for read", Arg1);
	break;
    default:
	sprintf(message, "Error");
    }

    mlen = strlen(message);
    XDrawImageString(display, output_win, gc, 4, owh - 10, message, mlen);

    XFlush(display);
    XSync(display, 1);

    sleep(2);

}
