/**
    FitsWindowControl.c: Window creation and handling

    Copyright 1992 Barry Solomon

    Permission to use, copy, modify, distribute this software is hereby
    granted without fee, provided that this Copyright notice appear in all
    copies.  It is provided "as is" without express or implied warranty.

    Revisions
    01/21/92	BDS	Created.
    03/03/92	BDS	Fix ImageCellWidth/ImageCellHeight calculations
**/

#include <stdio.h>

#include "FitsWindowDefs.h"
#include "FitsColorDefs.h"
#include "FitsDefs.h"
#include "FitsImageDefs.h"

char message[133];

/****************************************************** create_window */
Window create_window(parent, xpos, ypos, width, height, bwidth, Banner)
/**
    Create the main window
**/
    Window   parent;
    int      xpos, ypos;
    int      width, height;
    int      bwidth;
    char     *Banner;
{
    Window          NewWin;
    int             winfo_mask;
    XSizeHints      XHints;
    XWMHints        wmhints;
    XSetWindowAttributes winfo;
/*--------------------------------------------------------------------*/
    winfo.background_pixmap = ParentRelative;
    winfo.background_pixel = 0;
    winfo.border_pixmap = CopyFromParent;
    winfo.border_pixel = 1;
    winfo.bit_gravity = NorthWestGravity;
    winfo.win_gravity = NorthWestGravity;
    winfo.backing_store = WhenMapped;
    winfo.backing_planes = 0;
    winfo.save_under = True;
    winfo.event_mask = NoEventMask;
    winfo.do_not_propagate_mask = NoEventMask;
    winfo.override_redirect = False;
    winfo.colormap = mycmap;
    winfo_mask = CWEventMask | CWBackingStore
	| CWBorderPixel | CWBackPixel
	| CWColormap;

    NewWin = XCreateWindow(display, parent, xpos, ypos, width, height, bwidth,
			   dep, CopyFromParent, vis, winfo_mask, &winfo);

    /* Set up events for the window */
    event_mask = StructureNotifyMask | KeyPressMask
	| ButtonPressMask | ButtonReleaseMask
	| EnterWindowMask | LeaveWindowMask
	| ColormapChangeMask | ExposureMask;

    XSelectInput(display, NewWin, event_mask);

    /* Set window banner title */
    XStoreName(display, NewWin, Banner);

    /* Create a graphics context for the window and set the background
     * color */
    gc = XCreateGC(display, NewWin, 0, 0);
    XSetBackground(display, gc, 1);

    /* Allow input to the display window */
    wmhints.flags = InputHint;
    wmhints.input = True;
    XSetWMHints(display, NewWin, &wmhints);

/*
 *  Set the window manager hints as to our desires for the handling of our
 *  windows.
 */
    XHints.flags = USSize | USPosition;
    XHints.x = xpos;
    XHints.y = ypos;
    XHints.width = width;
    XHints.height = height;
    XSetNormalHints(display, NewWin, &XHints);

    /* Map the window to the display and flush the output queue */
    XMapWindow(display, NewWin);
    XFlush(display);

    return (NewWin);

}/* end create_window() */

/******************************************* update_graphics_window */
update_graphics_window( NumCols, NumRows)
    int	    NumRows, NumCols;
/**
    Resize the graphics window
**/
/**
    Resize the graphics window if either
	1. There is only one image and it's size is different
           than the old image.
        2. This is a secondary image and it is larger than the
           primary image.
**/
{
    XSizeHints      xsh;
/*------------------------------------------------------------*/
/*
 *  Modify ImageCellWidth and Height if the image should get too large
 */
    if ((NumRows <= 200) && (NumCols <= 200)) {
	ImageCellHeight = 5;
	ImageCellWidth = 5;
    }
    else if ((NumRows < 400) && (NumCols < 400)) {
	ImageCellHeight = 4;
	ImageCellWidth = 4;
    }
    else if ((NumRows < 600) && (NumCols < 600)) {
	ImageCellHeight = 2;
	ImageCellWidth = 2;
    }
    else if ((NumRows >= 600) || (NumCols >= 600)) {
	ImageCellHeight = 1;
	ImageCellWidth = 1;
    }

/*
 *  if the graphics window already exists then simply resize it
 */
    if (graph_win) {
/* 
 *      Calculate the size and postion of the graphics window 
 */
	gwh = NumRows * ImageCellHeight;
	gww = NumCols * ImageCellWidth;
	gwx2 = gwx1 + gww;
	gwy2 = gwy1 + gwh;

	xsh.width = gww;
	xsh.height = gwh;
	xsh.flags = USSize;
/*
 *      Resize the grphics window
 */
	XSetNormalHints(display, graph_win, &xsh);
	XResizeWindow(display, graph_win, gww, gwh);

    }

    else {
/*
 *     Calculate the size and postion of the graphics window
 */
	gwx1 = owx2 + 10;
	gwy1 = owy1;
	gwh = NumRows * ImageCellHeight;
	gww = NumCols * ImageCellWidth;
	gwx2 = gwx1 + gww;
	gwy2 = gwy1 + gwh;
/*
 *      Create the graphics window
 */
	graph_win = create_window(root, gwx1, gwy1, gww, gwh, 1, "FitsGraphics");
	XResizeWindow(display, graph_win, gww, gwh);

    }

}/* end create_graphics_window() */

/****************************************************** CreateCMapWin */
ErrorCode   CreateCMapWin()
{
/*--------------------------------------------------------------------*/

    CMapRows = (numcells > 254) ? 16 : 4;
    CMapCols = (numcells + CMapRows - 1) / CMapRows;
    banner_height = 25;

    xoff = 5;
    yoff = 5;

    if (CMapRows == 4) {
	CMapCellWidth = 50;
	CMapCellHeight = 40;
    }
    else {
	CMapCellWidth = (oww / 16);
	CMapCellHeight = 0.6 * CMapCellWidth;
    }

    MyWindows[COLOR].h = (CMapRows * CMapCellHeight) + banner_height;
    MyWindows[COLOR].w = (CMapCols * CMapCellWidth);

    MyWindows[COLOR].id = create_window(root, owx1, owy2 + 30,
			      MyWindows[COLOR].w, MyWindows[COLOR].h, 1,
			      "FitsColormap");

    UpdateCMapWin();

}/* CreateCMapWindow() */

/****************************************************** UpdateColorWin */
UpdateCMapWin()
{
    int             xpos, ypos, index;
    int             cur_row, cur_col;
    int             num;
/*--------------------------------------------------------------------*/
    /* Calculate an offset value so as to center the colortable grid in
     * the window */
    xoff = 0.45 * (MyWindows[COLOR].w - (CMapCols * CMapCellWidth));
    yoff = (MyWindows[COLOR].h - (CMapRows * CMapCellHeight));

    /* Fill a rectagle for each color in the colormap */
    for (cur_row = 0; cur_row < CMapRows; cur_row++) {
	for (cur_col = 0; cur_col < CMapCols; cur_col++) {

	    index = (cur_row * CMapCols) + cur_col;
	    if (index >= vis->map_entries)
		index = 0;

	    xpos = (cur_col * CMapCellWidth) + xoff;
	    ypos = (cur_row * CMapCellHeight) + yoff;

	    /* Draw a filled rectangle with interior color_table_index =
	     * index */
	    XSetForeground(display, gc, index);
	    XFillRectangle(display, MyWindows[COLOR].id, gc, xpos, ypos,
			   CMapCellWidth, CMapCellHeight);

	    /* Draw a black border around the filled polygon */
	    XSetForeground(display, gc, 0);
	    XDrawRectangle(display, MyWindows[COLOR].id, gc, xpos, ypos,
			   CMapCellWidth, CMapCellHeight);

	}	/* end for cur_col */
    }	/* end for cur_row */

    /* Flush the output buffer */
    XFlush(display);

}/* end UpdateColorWin() */
